@@ -28,7 +28,17 @@ sub preload {
 sub load_app {
     my($class, $app) = @_;
     return $apps{$app} ||= do {
-        local $ENV{MOD_PERL}; # trick Catalyst/CGI.pm etc.
+        # Trick Catalyst, CGI.pm, CGI::Cookie and others that check
+        # for $ENV{MOD_PERL}.
+        #
+        # Note that we delete it instead of just localizing
+        # $ENV{MOD_PERL} because some users may check if the key
+        # exists, and we do it this way because "delete local" is new
+        # in 5.12:
+        # http://perldoc.perl.org/5.12.0/perldelta.html#delete-local
+        local $ENV{MOD_PERL};
+        delete $ENV{MOD_PERL};
+
         Plack::Util::load_psgi $app;
     };
 }
@@ -40,16 +50,18 @@ sub call_app {
 
     my $env = {
         %ENV,
-        'psgi.version'        => [ 1, 1 ],
-        'psgi.url_scheme'     => ($ENV{HTTPS}||'off') =~ /^(?:on|1)$/i ? 'https' : 'http',
-        'psgi.input'          => $r,
-        'psgi.errors'         => *STDERR,
-        'psgi.multithread'    => Plack::Util::FALSE,
-        'psgi.multiprocess'   => Plack::Util::TRUE,
-        'psgi.run_once'       => Plack::Util::FALSE,
-        'psgi.streaming'      => Plack::Util::TRUE,
-        'psgi.nonblocking'    => Plack::Util::FALSE,
-        'psgix.harakiri'      => Plack::Util::TRUE,
+        'psgi.version'           => [ 1, 1 ],
+        'psgi.url_scheme'        => ($ENV{HTTPS}||'off') =~ /^(?:on|1)$/i ? 'https' : 'http',
+        'psgi.input'             => $r,
+        'psgi.errors'            => *STDERR,
+        'psgi.multithread'       => Plack::Util::FALSE,
+        'psgi.multiprocess'      => Plack::Util::TRUE,
+        'psgi.run_once'          => Plack::Util::FALSE,
+        'psgi.streaming'         => Plack::Util::TRUE,
+        'psgi.nonblocking'       => Plack::Util::FALSE,
+        'psgix.harakiri'         => Plack::Util::TRUE,
+        'psgix.cleanup'          => Plack::Util::TRUE,
+        'psgix.cleanup.handlers' => [],
     };
 
     if (defined(my $HTTP_AUTHORIZATION = $r->headers_in->{Authorization})) {
@@ -77,8 +89,22 @@ sub call_app {
         die "Bad response $res";
     }
 
-    if ($env->{'psgix.harakiri.commit'}) {
-        $r->child_terminate;
+    if (@{ $env->{'psgix.cleanup.handlers'} }) {
+        $r->push_handlers(
+            PerlCleanupHandler => sub {
+                for my $cleanup_handler (@{ $env->{'psgix.cleanup.handlers'} }) {
+                    $cleanup_handler->($env);
+                }
+
+                if ($env->{'psgix.harakiri.commit'}) {
+                    $r->child_terminate;
+                }
+            },
+        );
+    } else {
+        if ($env->{'psgix.harakiri.commit'}) {
+            $r->child_terminate;
+        }
     }
 
     return Apache2::Const::OK;
